from fastapi.testclient import TestClient
from typing import List
import pytest
from ..main import app
from ..structs.landmark import Landmark


@pytest.fixture()
def client():
    return TestClient(app)


# Base test for checking if the API returns correct error code when no preferences are specified.
def test_new_trip_invalid_prefs(client):
    response = client.post(
        "/trip/new",
        json={
            "preferences": {},
            "start": [48.8566, 2.3522]
            }
        )
    assert response.status_code == 422

    
# Test no. 1
def test_turckheim(client):
    duration_minutes = 15
    response = client.post(
        "/trip/new",
        json={
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 5}, "nature": {"type": "nature", "score": 5}, "shopping": {"type": "shopping", "score": 5}, "max_time_minute": duration_minutes, "detour_tolerance_minute": 0},
            "start": [48.084588, 7.280405]
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert isinstance(landmarks, list)  # check that the return type is a list
    assert duration_minutes*0.8 < int(result['total_time']) < duration_minutes*1.2
    assert len(landmarks) > 2           # check that there is something to visit


# Test no. 2
def test_bellecour(client) :
    duration_minutes = 35
    response = client.post(
        "/trip/new",
        json={
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 5}, "nature": {"type": "nature", "score": 5}, "shopping": {"type": "shopping", "score": 5}, "max_time_minute": duration_minutes, "detour_tolerance_minute": 0},
            "start": [45.7576485, 4.8330241]
            }
        )
    result = response.json()
    landmarks = load_trip_landmarks(client, result['first_landmark_uuid'])
    osm_ids = landmarks_to_osmid(landmarks)

    # checks :
    assert response.status_code == 200  # check for successful planning
    assert duration_minutes*0.8 < int(result['total_time']) < duration_minutes*1.2
    assert 136200148 in osm_ids         # check for Cathédrale St. Jean in trip



def landmarks_to_osmid(landmarks: List[Landmark]) -> list :
    """
    Convert the list of landmarks into a list containing their osm ids for quick landmark checking.
    
    Args :
        landmarks (list): the list of landmarks
    
    Returns :
        ids (list)      : the list of corresponding OSM ids
    """
    ids = []
    for landmark in landmarks :
        ids.append(landmark.osm_id)

    return ids

def fetch_landmark(client, landmark_uuid):
    """
    Fetch landmark data from the API based on the landmark UUID.

    Args:
        landmark_uuid (str): The UUID of the landmark.

    Returns:
        dict: Landmark data fetched from the API.
    """
    response = client.get(f"/landmark/{landmark_uuid}")

    if response.status_code != 200:
        raise Exception(f"Failed to fetch landmark with UUID {landmark_uuid}: {response.status_code}")
    
    json_data = response.json()
    
    if "detail" in json_data:
        raise Exception(json_data["detail"])
    
    return json_data


def load_trip_landmarks(client, first_uuid):
    """
    Load all landmarks for a trip using the response from the API.

    Args:
        first_uuid (str) : The first UUID of the landmark.

    Returns:
        landmarks (list) : An list containing all landmarks for the trip.
    """
    landmarks = []
    next_uuid = first_uuid

    while next_uuid is not None:
        landmark_data = fetch_landmark(client, next_uuid)
        landmarks.append(Landmark(**landmark_data)) # Create Landmark objects
        next_uuid = landmark_data.get('next_uuid')  # Prepare for the next iteration

    return landmarks




# def test_new_trip_single_prefs(client):
#     response = client.post(
#         "/trip/new",
#         json={
#             "preferences": {"sightseeing": {"type": "sightseeing", "score": 1}, "nature": {"type": "nature", "score": 1}, "shopping": {"type": "shopping", "score": 1}, "max_time_minute": 360, "detour_tolerance_minute": 0},
#             "start": [48.8566, 2.3522]
#             }
#         )
#     assert response.status_code == 200


# def test_new_trip_matches_prefs(client):
#     # todo
#     pass
